package com.guwan.backend.jpaTest;

import com.guwan.backend.builder.JoinBuilder;
import com.guwan.backend.jpaTest.domain.Department;
import com.guwan.backend.jpaTest.domain.Employee;
import com.guwan.backend.jpaTest.domain.EmployeeDTO;
import com.guwan.backend.jpaTest.domain.Project;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class EmployeeService {

    @Autowired
    private EmployeeRepository employeeRepository;
    
    @PersistenceContext
    private EntityManager entityManager;

    /**
     * 获取员工详细信息，包含部门和项目信息
     * 使用字符串方式指定字段，并直接映射到DTO
     * 
     * @param departmentName 部门名称过滤条件
     * @param projectDeleted 项目是否已删除的过滤条件
     * @return 员工DTO列表
     */
    public Page<EmployeeDTO> getEmployeeDetails(String departmentName, boolean projectDeleted,
                                                int page, int size) {
        // 创建查询构建器，从Employee实体开始
        JoinBuilder<Employee> builder = JoinBuilder.from(Employee.class);

        // 添加关联，使用字符串指定实体间的关系字段
        builder.join(Department.class, "departmentId", "id");
        builder.join(Project.class, "projectId", "id");

        // 添加查询条件，使用字符串指定字段
        if (departmentName != null && !departmentName.isEmpty()) {
            builder.equal(Department.class, "name", departmentName);
        }
        builder.equal(Project.class, "deleted", projectDeleted);

        // 使用selectAs设置字段别名，避免使用FieldMapping
        builder.selectAs("employeeId", Employee.class, "id");
        builder.selectAs("employeeName", Employee.class, "name");
        builder.selectAs("departmentName", Department.class, "name");
        builder.selectAs("projectName", Project.class, "name");

        // 先获取总记录数
        long total = builder.count(entityManager);

        // 执行分页查询并映射到DTO
        List<EmployeeDTO> content = builder.executeAndMapWithPaging(
                entityManager,
                EmployeeDTO.class,
                page,
                size
        );

        // 返回分页结果
        return new PageImpl<>(content, PageRequest.of(page, size), total);
    }

} 